{***********************************************************
 Component: TDBTreeView version 1.04

 Author: Didier Cabal
         'Bellefont'
         31800 Saint-Gaudens, France
         tl: +33 (0)5 61 89 72 92
         e-mail: didier.cabale+delphi@gmail.com
         web-page: http://didier.cabale.free.fr/delphi.htm

 Special Thanks to RALib authors:
         Andrey Prigunkov   (black@infa.ru, blacknbs@chat.ru)
         Roman  Tkachev     (roman@infa.ru, whiteman@mail.ru)
************************************************************}

unit DBTreeView;

{$Resource *.res}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Dialogs, Graphics,
  CommCtrl, ComCtrls, ExtCtrls, Db, DBTables, dbctrls, Variants;

type

{ Forward declaration }
  TDBTreeNode = class;
  TTVDataLink = class;

  TCustomDBTreeView = class(TCustomTreeView)
  private
    InDataScrolled : Boolean;
    InDelete : Boolean;
    InAddChild : Boolean;
    InRecordChange: Boolean;
    FDepth : byte;
    oldRecCount : integer;
    oldIndexFieldNames: string;
    InRefreshChild : boolean;
    InVKInsert : boolean;
    Sel : TTreeNode;
    FDataLink : TTVDataLink;
    FFieldString : String;
    FFieldList: TStringList;
    procedure SetFieldListFromDS;
    procedure InternalDataSetChanged;
    procedure InternalDataSetScrolled;
    procedure InternalRecordChanged(Field: TField);
    procedure CheckDataSet;
    function FindNextNode(const Node : TTreeNode) : TTreeNode;
    function CanExpand(Node: TTreeNode) : Boolean; override;
    function AddChildNode(const Node : TTreeNode; const Select : boolean) : TDBTreeNode;
    function CanEdit(Node: TTreeNode): Boolean; override;
    procedure Edit(const Item: TTVItem); override;
    procedure DeleteNode(Node : TTreeNode);
    procedure HideEditor;
    function CreateNode: TTreeNode; override;
    function FindNode(ANodeValue : variant) : TDBTreeNode;
    function FindFinalNode(ANodeValue : Variant): TDBTreeNode;
    procedure Notification(Component : TComponent; Operation : TOperation); override;
    procedure Change(Node: TTreeNode); override;
    procedure Collapse(Node: TTreeNode); override;
    function ValidDataSet: Boolean;
    procedure LinkActive(Value: Boolean); virtual;
    procedure RefreshChild(ANode : TDBTreeNode);
    function GetDataSource: TDataSource;
    procedure SetDataSource(const Value: TDataSource);
    procedure SetFieldString(const Value: String);
  protected
    procedure DataSetChanged; dynamic;
    procedure DataSetScrolled; dynamic;
    procedure Change2(Node: TTreeNode); dynamic;
    procedure RecordChanged(Field: TField); dynamic;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DataLink: TTVDataLink read FDataLink;
    property FieldString: String read FFieldString write SetFieldString;
  end;

  TTVDataLink = class(TDataLink)
  private
    FTreeView: TCustomDBTreeView;
  protected
    procedure ActiveChanged; override;
    procedure RecordChanged(Field: TField); override;
    procedure DataSetChanged; override;
    procedure DataSetScrolled(Distance: Integer); override;
  public
    constructor Create(ATreeView: TCustomDBTreeView);
    destructor Destroy; override;
  end;

  TDBTreeNode = class(TTreeNode)
  private
    FNodeValue: Variant;
  public
    property NodeValue: Variant read FNodeValue;
  end;

  TDBTreeView = class(TCustomDBTreeView)
  published
    property DataSource;
    property BorderStyle;
    property DragCursor;
    property ShowButtons;
    property ShowLines;
    property ShowRoot;
    property ReadOnly;
    property RightClickSelect;
    property DragMode;
    property HideSelection;
    property Indent;
    property OnEditing;
    property OnEdited;
    property OnExpanding;
    property OnExpanded;
    property OnCollapsing;
    property OnCompare;
    property OnCollapsed;
    property OnChanging;
    property OnChange;
    property OnDeletion;
    property OnGetImageIndex;
    property OnGetSelectedIndex;
    property Align;
    property Enabled;
    property Font;
    property Color;
    property ParentColor default False;
    property ParentCtl3D;
    property Ctl3D;
    property SortType;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnClick;
    property OnEnter;
    property OnExit;
    property OnDragDrop;
    property OnDragOver;
    property OnStartDrag;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property PopupMenu;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property Images;
    property StateImages;
    property Anchors;
    property AutoExpand;
    property BiDiMode;
    property BorderWidth;
    property ChangeDelay;
    property Constraints;
    property DragKind;
    property HotTrack;
    property ParentBiDiMode;
    property RowSelect;
    property ToolTips;
    property OnCustomDraw;
    property OnCustomDrawItem;
    property OnEndDock;
    property OnStartDock;
    property FieldString;
  end;

  EDBTreeViewError = class(ETreeViewError);

procedure Register;

implementation

{ TCustomDBTreeView }

const
  SDeleteNode             = 'Delete %s ?';
  SDeleteNode2            = 'Delete %s (with its children)?';
  SDataSetNotActive       = 'DataSet not active';

function TCustomDBTreeView.ValidDataSet: Boolean;
var
   i: byte;
begin
  Result := FDataLink.Active and Assigned(FDataLink.DataSet)
{pourquoi tant de conditions?}
   {and FDataLink.DataSet.Active};
{check if FFieldList has already been populated by the SetFieldString method}
//  if Result and (FFieldString = '') then// begin
  if Result and (FFieldList.Count = 0) then// begin
{     for i := 0 to FDataLink.DataSet.FieldCount -1 do
         FFieldList.Add(IntToStr(i));
     FDepth := FFieldList.Count;}
     SetFieldListFromDS;
//  end
end;

procedure TCustomDBTreeView.LinkActive(Value: Boolean);
begin
  if not Value then HideEditor;
  //if ( csDesigning in ComponentState ) then Exit;
  if ValidDataSet then begin
    RefreshChild(nil);
    oldRecCount := FDataLink.DataSet.RecordCount;
  end else {if FUpdateLock = 0 then}
    Items.Clear;
end;

procedure TCustomDBTreeView.RefreshChild(ANode: TDBTreeNode);

  function NodeLevel(ANode: TDBTreeNode): shortint;
  begin
     if ANode = nil then
        Result := -1
     else
        Result := ANode.Level;
  end;

  function PVAsString(AParentValue: Variant): string;
  begin
     if AParentValue = Null then
        result := 'null'
     else
        result := ''''+VarToStr(AParentValue)+'''';
  end;

  function GetFieldNo(NL: shortint): shortint;
  begin
     result := NL;
     if (NL > -1) and (FFieldList.Count > 0) then
//        try
        result := StrToInt(FFieldList[NL]);
//        except end;
  end;

  function GetField(aDataSet: TDataSet; FieldNo: byte): TField;
  begin
      with aDataSet do begin
         if Fields[FieldNo].FieldKind = fkLookup then
            result := FieldByName(Fields[FieldNo].KeyFields)
         else
            result := Fields[FieldNo];
      end
  end;

var
   ParentValue, BrotherValue : variant;
   BK : TBookmark;
   oldFilter : string;
   oldFiltered :boolean;
   PV : string;
   i: integer;
   NL, FieldNo2, FieldNo3: shortint;
   ParentNode: TDBTreeNode;
   Field1: TField;
begin
  with FDataLink.DataSet do begin
  BK := GetBookmark;
  if InRefreshChild then exit;
  InRefreshChild := true;
  try
    DisableControls;
    NL := NodeLevel(ANode);
    ParentNode := ANode;
    BrotherValue := Null;
    if ANode <> nil then begin
      ANode.DeleteChildren;
      ParentValue := ANode.FNodeValue;
      ANode.ImageIndex := 1;
      ANode.SelectedIndex := ANode.ImageIndex;
    end else begin
      Items.Clear;
      ParentValue := Null;
    end;
    oldFilter := Filter; oldFiltered := Filtered;
    {this to avoid a filter if there's no parent}
    if NL >= 0 then begin
       FieldNo2 := GetFieldNo(NL);
{       if Fields[FieldNo2].FieldKind = fkLookup then
          Field1 := FieldByName(Fields[FieldNo2].KeyFields)
       else
          Field1 := Fields[FieldNo2];}
       Field1 := GetField(FDataLink.DataSet, FieldNo2);
       if Filtered then
          Filter := '('+Field1.FieldName+'='+PVAsString(ParentValue)+') and '+Filter
       else
       {modifier le code ici pour viter une exception si utilisation d'1 champ calcul
       }
          Filter := '('+Field1.FieldName+'='+PVAsString(ParentValue)+')';
       for i := NL-1 downto 0 do begin
          FieldNo2 := GetFieldNo(i);
          ParentNode := TDBTreeNode(ParentNode.Parent);
          ParentValue := ParentNode.FNodeValue;
{          if (Fields[FieldNo2].FieldKind = fkLookup) then
             Field1 := FieldByName(Fields[FieldNo2].KeyFields)
          else
             Field1 := Fields[FieldNo2];}
          Field1 := GetField(FDataLink.DataSet, FieldNo2);
          if (Filter <> '') then
             Filter := '('+Field1.FieldName+'='+PVAsString(ParentValue)+') and '+Filter
          else
             Filter := '('+Field1.FieldName+'='+PVAsString(ParentValue)+')';
       end;
       Filtered := true;
    end;
    FieldNo3 := GetFieldNo(NL+1);
{    if Fields[FieldNo3].FieldKind = fkLookup then
       Field1 := FieldByName(Fields[FieldNo3].KeyFields)
    else
       Field1 := Fields[FieldNo3];}
    Field1 := GetField(FDataLink.DataSet, FieldNo3);
    try
       First;
       while not Eof do begin
          if (BrotherValue <> Field1.AsVariant) or (BrotherValue = null) then
          with Items.AddChild(ANode, Fields[FieldNo3].asstring) as TDBTreeNode do begin
             FNodeValue := Field1.AsVariant;
             BrotherValue := FNodeValue;
          end;
          Next;
       end;
    finally
       Filtered := oldFiltered; Filter := oldFilter;
    end;
    if ANode = nil then
        for i := 0 to Items.Count-1 do
          with Items[i] as TDBTreeNode do begin
            HasChildren := Level +1 < FFieldList.Count;
            if HasChildren then
               ImageIndex := 0
            else
               ImageIndex := 2;
          end
    else
        for i := 0 to ANode.Count-1 do
          with ANode[i] as TDBTreeNode do begin
            HasChildren := Level +1 < FFieldList.Count;
            if HasChildren then
               ImageIndex := 0
            else
               ImageIndex := 2;
            SelectedIndex := ImageIndex;
          end;
  finally
  try
{condition indispensable, sinon une erreur est gnre, car le bookmark ne renvoie
pas d'enregistrement correct si le dataset est en dsEdit.
comment fixer un bookmark en state dsEdit?}
     if not InRecordChange then
        GotoBookmark(BK);
     FreeBookmark(BK);
     EnableControls;
     InRefreshChild := false;
  finally
//        dec(FUpdateLock);
  end; {of try}
  end  {of try}
  end; {of with}
end;

procedure TCustomDBTreeView.HideEditor;
begin
  if Selected <> nil then
    Selected.EndEdit(true);
end;

procedure TCustomDBTreeView.Collapse(Node: TTreeNode);
var
  HasChildren : boolean;
begin
  inherited Collapse(Node);
{  if not FPersistentNode then begin}
    HasChildren := Node.HasChildren;
    Node.DeleteChildren;
    Node.HasChildren := HasChildren;
    Node.ImageIndex := 0;
    Node.SelectedIndex := Node.ImageIndex;
//  end;
end;

constructor TCustomDBTreeView.Create(AOwner: TComponent);
var
   i: byte;
begin
  inherited Create(AOwner);
  FDataLink := TTVDataLink.Create(Self);
  FFieldList := TStringList.Create;
end;

destructor TCustomDBTreeView.Destroy;
begin
  FDataLink.Free;
  FFieldList.Free;
  inherited Destroy;
end;

procedure TCustomDBTreeView.Change(Node: TTreeNode);
begin
//  if ValidDataSet and Assigned(Node){ and not InDataScrolled and}
  if Assigned(Node){ and not InDataScrolled and}
{  if ValidDataSet and (Node.Level = FDepth -1){ and not InDataScrolled and
    (FUpdateLock = 0)} and
    (FDataLink.DataSet.State in [dsBrowse, dsEdit, dsInsert])
  then
    if (Node.Level = FDepth -1) then begin
//    inc(FUpdateLock);
    try
       ReadOnly := false;
       Change2(Node);
    finally
//      dec(FUpdateLock);
    end;
    end else
       ReadOnly := true;
  inherited Change(Node);
end;

function TCustomDBTreeView.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TCustomDBTreeView.Notification(Component: TComponent;
  Operation: TOperation);
begin
  inherited Notification(Component, Operation);
  if (FDataLink <> nil) and (Component= DataSource) and (Operation = opRemove) then
    DataSource := nil;
end;

procedure TCustomDBTreeView.SetDataSource(const Value: TDataSource);
begin
  if Value = FDatalink.DataSource then Exit;
  Items.Clear;
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

procedure TCustomDBTreeView.DataSetChanged;
var
  RecCount : integer;
  IndexFieldNames: string;
begin
  case FDataLink.DataSet.State of
    dsBrowse : begin
        RecCount := FDataLink.DataSet.RecordCount;
        if FDataLink.DataSet is TTable then
           IndexFieldNames := TTable(FDataLink.DataSet).IndexFieldNames;
        if {(RecCount = -1) or} (RecCount <> oldRecCount) and not InVKInsert and not Indelete
           or (IndexFieldNames <> oldIndexFieldNames) then
{         voir si ci-dessous tient:}
//        if {(RecCount = -1) or (RecCount <> oldRecCount) and} not InVKInsert and not Indelete then
//          UpdateTree;
          RefreshChild(nil);
        oldRecCount := RecCount;
        oldIndexFieldNames := IndexFieldNames;
    end;
    dsInsert : oldRecCount := -1; { TQuery don't change RecordCount value after insert new record }
  end;
  Selected := FindFinalNode(FDataLink.DataSet.Fields[StrToInt(FFieldList[FFieldList.Count - 1])].AsVariant);
end;

function TCustomDBTreeView.FindNode(ANodeValue: variant): TDBTreeNode;
var
  i : integer;
begin
  for i := 0 to Items.Count -1 do begin
     Result := Items[i] as TDBTreeNode;
     if VarToStr(Result.FNodeValue) = VarToStr(ANodeValue) then
        exit;
  end;
  Result := nil;
end;

function TCustomDBTreeView.FindFinalNode(ANodeValue: Variant): TDBTreeNode;
var
  i : integer;
begin
  for i := 0 to Items.Count -1 do
    if items[i].Level = FDepth - 1 then begin
       Result := Items[i] as TDBTreeNode;
       if VarToStr(Result.FNodeValue) = VarToStr(ANodeValue) then
          exit;
    end;
  Result := nil;
end;

function TCustomDBTreeView.CreateNode: TTreeNode;
begin
  Result := TDBTreeNode.Create(Items);
end;

procedure TCustomDBTreeView.InternalDataSetChanged;
begin
  if not HandleAllocated {or UpdateLocked }or InDataScrolled then Exit;
  try
    DataSetChanged;
  finally
  end;
end;

procedure TCustomDBTreeView.InternalDataSetScrolled;
begin
  if not HandleAllocated {or UpdateLocked }then Exit;
  InDataScrolled := true;
  try
    DataSetScrolled;
  finally
    InDataScrolled := false;
  end;
end;

procedure TCustomDBTreeView.InternalRecordChanged(Field: TField);
begin
  if not (HandleAllocated and ValidDataSet) then Exit;
    if {(Selected <> nil) and (FUpdateLock = 0) and}
       (FDataLink.DataSet.State = dsEdit) then begin
       try
          RecordChanged(Field);
       finally
//      dec(FUpdateLock);
       end;
    end;
end;

procedure TCustomDBTreeView.Change2(Node: TTreeNode);
begin
    with FDataLink.DataSet do
       Locate(Fields[strtoint(FFieldList[FFieldList.Count - 1])].FieldName, (Node as TDBTreeNode).FNodeValue, []);
end;

procedure TCustomDBTreeView.DataSetScrolled;
var
   ANode: TTreeNode;
begin
   ANode := FindFinalNode(FDataLink.DataSet.Fields[strtoint(FFieldList[FFieldList.Count - 1])].AsVariant);
      Selected := ANode;
end;

procedure TCustomDBTreeView.RecordChanged(Field: TField);
var
  Node : TDBTreeNode;
begin
  InRecordChange := true;
  if Assigned(Field) then
     if (Field.Index > 0) then begin
        Node := FindNode(FDataLink.DataSet.Fields[Field.Index -1].AsVariant);
        if not (Node = Nil) then
           Collapse(Node);
     end else
        RefreshChild(nil);
{  with (Selected as TRADBTreeNode) do
    if FIconField <> '' then begin
      ImageIndex := Var2Type(FDataLink.DataSet[FIconField], varInteger);
      SelectedIndex := ImageIndex + FSelectedIndex;
    end;}
  InRecordChange := false;
end;

procedure TCustomDBTreeView.Edit(const Item: TTVItem);
var
   Node: TDBTreeNode;
begin
  CheckDataSet;
  inherited Edit(Item);
  if Assigned(Selected) then begin
//    inc(FUpdateLock);
    try
      if (Item.pszText <> nil) and (FDataLink.Edit) then begin
         FDataLink.DataSet.Fields[FDepth-1].AsString := Item.pszText;
         FDatalink.DataSet.Post;
      end else
        FDataLink.DataSet.Cancel;
    finally
{      dec(FUpdateLock);}
    end;
  end;
end;

procedure TCustomDBTreeView.CheckDataSet;
begin
  if not ValidDataSet then
    raise EDBTreeViewError.Create(SDataSetNotActive);
end;

procedure TCustomDBTreeView.DeleteNode(Node: TTreeNode);
var
  NewSel : TTreeNode;
begin
  CheckDataSet;
//  inc(FUpdateLock);
  InDelete := true;
  try
    NewSel := FindNextNode(Selected);
    FDataLink.DataSet.Delete;
    Node.Free;
    if NewSel <> nil then
      NewSel.Selected := true;
  finally
    InDelete := false;
//    dec(FUpdateLock);
  end;
end;

function TCustomDBTreeView.FindNextNode(const Node: TTreeNode): TTreeNode;
begin
  if (Node <> nil) and (Node.Parent <> nil) then
    if Node.Parent.Count > 1 then
      if Node.Index = Node.Parent.Count-1 then
        Result := Node.Parent[Node.Index-1] else
        Result := Node.Parent[Node.Index+1]
    else
      Result := Node.Parent
  else
      Result := nil;
end;

procedure TCustomDBTreeView.KeyDown(var Key: Word; Shift: TShiftState);
  procedure DeleteSelected;
  var
    M : string;
  begin
    if Selected.HasChildren then M := SDeleteNode2 else M := SDeleteNode;
    if MessageDlg(Format(M, [Selected.Text]), mtConfirmation, [mbYes, mbNo], 0) = mrYes then
      DeleteNode(Selected);
  end;
begin
  inherited KeyDown(Key, Shift);
  if not ValidDataSet or (FDataLink.ReadOnly) or ReadOnly then exit;
  case Key of
    VK_DELETE :
      if ([ssCtrl] = Shift) and Assigned(Selected) then
          DeleteSelected;
    VK_INSERT :
      if not IsEditing then begin
         InVKInsert := true;
         Sel := Selected;
{        if [ssAlt] = Shift then
          //AddChild
          AddChildNode(Selected, true).EditText
        else}
          //Add
         try
            AddChildNode(Selected.Parent, true).EditText;
         finally
            InVKInsert := false;
         end;
      end;
    VK_F2 : if Selected <> nil then Selected.EditText;
  end;
end;

function TCustomDBTreeView.AddChildNode(const Node: TTreeNode;
  const Select: boolean): TDBTreeNode;
var
//  MV : variant;
  M : string;
  PN : TDBTreeNode;
  i: byte;
begin
  CheckDataSet;
  try
    FDataLink.DataSet.Append;
    if FDataLink.DataSet.Fields[FDepth-1].AsVariant = null then
    begin
       PN := (Node as TDBTreeNode);
       M := '';
       FDataLink.DataSet.Fields[FDepth -2].AsVariant := PN.FNodeValue;
       for i := FDepth -3 downto 0 do begin
           PN := (PN.Parent as TDBTreeNode);
           FDataLink.DataSet.Fields[i].AsVariant := PN.FNodeValue;;
       end;
    end else
       M := FDataLink.DataSet.Fields[FDepth-1].AsString;
    Result := Items.AddChild(Node, M) as TDBTreeNode;
    with Result do begin
      if Result.Level = FDepth - 1 then
      FNodeValue := FDataLink.DataSet.Fields[FDepth-1].AsVariant;
{      if FIconField <> '' then
      begin
        ImageIndex := Var2Type(FDataLink.DataSet[FIconField], varInteger);
        SelectedIndex := ImageIndex + FSelectedIndex;
      end;}
    end;
    Result.Selected := Select;
  finally
//    dec(FUpdateLock);
  end;
end;

function TCustomDBTreeView.CanExpand(Node: TTreeNode): Boolean;
var
   selNode: TTreeNode;
   ANode: TDBTreeNode;
begin
   selNode := Selected;
   Result := inherited CanExpand(Node);
   ANode := nil;
   if Result and (Node.Count = 0) then begin
      if FDataLink.DataSet.State = dsEdit then
         FDataLink.DataSet.Post;
      RefreshChild(Node as TDBTreeNode);
      ANode := FindNode(FDataLink.DataSet.Fields[FDataLink.DataSet.FieldCount -1].AsVariant);
   end;
   if ANode = nil then
      Selected := selNode
   else Selected := (ANode as TTreeNode);
end;

function TCustomDBTreeView.CanEdit(Node: TTreeNode): Boolean;
var
   ReadOnly: boolean;
begin
  Result := inherited CanEdit(Node);
  ReadOnly := true;
  if FDataLink.DataSet <> nil then
{pourquoi quand le dataset est  readonly true, le datalink ne suit pas la mme proprit?}
{ancienne procdure:
   Result := Result and not FDataLink.ReadOnly;}
     if FDataLink.DataSet is TTable then
        ReadOnly := (FDataLink.DataSet as TTable).ReadOnly
     else
     if FDataLink.DataSet is TQuery then
        ReadOnly := not (FDataLink.DataSet as TQuery).RequestLive
     else
        ReadOnly := true;
     Result := Result and not ReadOnly ;
end;

{this not object linked procedure because if belonging to a TDBTreeView, the
delete(S, index, count) procedure is tried to be applied to a TDBTreeView}
procedure DeleteString(var S: String; index, count: integer);
begin
   delete(S, index, count);
end;

procedure TCustomDBTreeView.SetFieldString(const Value: String);
var
   S: string;
   NbCarToDelete, NbCarToAdd, i, j: byte;
begin
{pour DC: rajouter une condition sur validdataset}
  FFieldList.Clear;
//  if ValidDataSet then begin
//  if Assigned(FDataLink.DataSet) then begin
    FFieldString := Value;
    S := Value;
//  while Length(S) > 0 do begin
    while Length(S) > 0 do begin
      if pos(';', S) > 0 then begin
        NbCarToDelete := pos(';', S);
        NbCarToAdd := NbCarToDelete -1;
      end else begin
        NbCarToDelete := Length(S);
        NbCarToAdd := NbCarToDelete;
      end;
      FFieldList.Add(copy(S, 1, NbCarToAdd));
//    showmessage(FFieldList[FFieldList.Count-1]);
      DeleteString(S, 1, NbCarToDelete);
    end;
    FDepth := FFieldList.Count;
{control the validity of strings}
    if FFieldList.Count > 0 then
      for i := 0 to FFieldList.Count -1 do
        try
           j := StrToInt(FFieldList[i]);
        except
           on E: EConvertError do
              ShowMessage(E.ClassName + E.Message);
        end
    else
      SetFieldListFromDS;
{this to make react the TDBTreeview with on FieldString changing}
    if ValidDataSet then
       RefreshChild(nil);
end;

procedure TCustomDBTreeView.SetFieldListFromDS;
var
   i: byte;
begin
//     FFieldList.Clear;
     for i := 0 to FDataLink.DataSet.FieldCount -1 do
         FFieldList.Add(IntToStr(i));
     FDepth := FFieldList.Count;
end;


{ TTVDataLink }

procedure TTVDataLink.ActiveChanged;
begin
  FTreeView.LinkActive(Active);
end;

constructor TTVDataLink.Create(ATreeView: TCustomDBTreeView);
begin
  inherited Create;
  FTreeView := ATreeView;
end;

{proc presque inutile}
procedure TTVDataLink.DataSetChanged;
begin
  FTreeView.InternalDataSetChanged;
end;

procedure TTVDataLink.DataSetScrolled(Distance: Integer);
begin
  FTreeView.InternalDataSetScrolled;
end;

destructor TTVDataLink.Destroy;
begin
  inherited Destroy;
end;

procedure TTVDataLink.RecordChanged(Field: TField);
begin
  FTreeView.InternalRecordChanged(Field);
end;

procedure Register;
begin
  RegisterComponents('DCComp', [TDBTreeView]);
end;

end.
